// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UIRichTextLabel.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

UIRichTextLabel::UIRichTextLabel(SceneItem *item) :
  UILabel(item)
{

}

void UIRichTextLabel::onStart() {
  UILabel::onStart();

  useEffect([&]{
    std::vector<struct UILabelStyle> styleStack;
    struct UILabelStyle current;
    styleStack.push_back(current);
    std::vector<struct UILabelText> bufferTexts;
    
    if(this->richText._realValue.empty()) {
      this->rebufferQuads(bufferTexts);
      return;
    }

    std::function<void(Xml*)> parseChildren = [&](Xml *node) {
      auto itChildren = node->childNodes.begin();
      while(itChildren != node->childNodes.end()) {
        auto child = *itChildren;
        if(child.nodeType == XML_NODE_TYPE_TEXT) {
          struct UILabelText text;
          text.style = current;
          text.text = child.value;
          bufferTexts.push_back(text);
        } else if(child.nodeType == XML_NODE_TYPE_ELEMENT) {
          auto node = child.child;
          assertTrue(node->node == "font");

          struct UILabelStyle style;
          if(node->attributes.contains("font")) {
            style.font = this->getGame()->assetManager.get<TrueTypeAsset>(node->attributes["font"]);
          } else {
            style.font = current.font;
          }

          if(node->attributes.contains("size")) {
            style.size = std::stoi(node->attributes["size"]);
          } else {
            style.size = current.size;
          }

          if(node->attributes.contains("style")) {
            std::string s = node->attributes["style"];
            style.style = 0;
            if(s.find("bold") != std::string::npos) style.style |= TRUE_TYPE_VARIANT_BOLD;
            if(s.find("italic") != std::string::npos) style.style |= TRUE_TYPE_VARIANT_ITALICS;
          } else {
            style.style = current.style;
          }

          if(node->attributes.contains("color")) {
            style.color = Color::fromString(node->attributes["color"]);
          } else {
            style.color = current.color;
          }

          styleStack.push_back(style);
          current = style;
          parseChildren(node);
          styleStack.pop_back();
          current = styleStack.back();
        }
        ++itChildren;
      }
    };

    auto root = Xml::load("<root><font font=\"font_main\">" + ((std::string)this->richText) + "</font></root>");
    parseChildren(&root);
    this->rebufferQuads(bufferTexts);
  }, this->richText)();
}